--[[
************************************************************************
Project				: Broker_TradeCooldowns
Author				: zhinjio
Project Revision	: r70
Project Date		: 20100916072150

File				: Broker_TradeCooldowns.lua
Commit Author		: zhinjio
Commit Revision		: 70
Commit Date			: 20100916072150
************************************************************************
Description	:
	Main addon file. 

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "BTC"
local FULLNAME = "Broker_TradeCooldowns"

local BTC = LibStub( "AceAddon-3.0" ):NewAddon( MODNAME, "AceEvent-3.0" )

local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )
local AceConfig 		= LibStub("AceConfig-3.0")
local AceConfigReg 		= LibStub("AceConfigRegistry-3.0")
local AceConfigDialog 	= LibStub("AceConfigDialog-3.0")

_G[ MODNAME ] = BTC
local addon = BTC

local addonversion = "r70"
local addonauthor = "zhinjio"
local builddate = "20100916072150"

local db

local tooltip
local LTT = LibStub:GetLibrary( "LibQTip-1.0" )

local ForcedScan = false
local TradeSkillOpen = false

function addon.Print( msg )
	local _, fh = DEFAULT_CHAT_FRAME:GetFont()
	fh = math.ceil( fh )
	local icon = "\124TInterface\\Addons\\" .. FULLNAME .. "\\icon:" .. fh .. "\124t"
	if ( msg ~= nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( icon .. " " .. MODNAME .. " : " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function addon.Debug( msg )
	if ( msg ~= nil ) then
		if ( db.options.debug ) then
			addon.Print( "DEBUG: " .. msg )
		end
	end
end

-- coloring functions for tooltip text
local function Red( text )		return "|cffff0000" .. text .. "|r" end
local function Yellow( text )	return "|cffffff00" .. text .. "|r" end
local function Green( text )	return "|cff00ff00" .. text .. "|r" end
local function White( text )	return "|cffffffff" .. text .. "|r" end

local primarySkills = {
	[L["Alchemy"]] = true,
	[L["Blacksmithing"]] = true,
	[L["Enchanting"]] = true,
	[L["Engineering"]] = true,
	[L["Herbalism"]] = true,
	[L["Inscription"]] = true,
	[L["Jewelcrafting"]] = true,
	[L["Leatherworking"]] = true,
	[L["Mining"]] = true,
	[L["Skinning"]] = true,
	[L["Tailoring"]] = true,
}

local secondarySkills = {
	[L["First Aid"]] = true,
	[L["Cooking"]] = true,
	[L["Fishing"]] = true,
	[L["Riding"]] = true,
}

local weaponSkills = {
	[L["Axes"]] = true,
	[L["Bows"]] = true,
	[L["Crossbows"]] = true,
	[L["Daggers"]] = true,
	[L["Defense"]] = true,
	[L["Maces"]] = true,
	[L["Guns"]] = true,
	[L["Polearms"]] = true,
	[L["Staves"]] = true,
	[L["Swords"]] = true,
	[L["Thrown"]] = true,
	[L["Two-Handed Axes"]] = true,
	[L["Two-Handed Maces"]] = true,
	[L["Two-Handed Swords"]] = true,
	[L["Unarmed"]] = true,
	[L["Wands"]] = true,
}

-- professions with cooldowns
local PROF_ENCHANTING		= GetSpellInfo(7411)
local PROF_INSCRIPTION		= GetSpellInfo(45357)
local PROF_JEWELCRAFTING	= GetSpellInfo(25229)
local PROF_TAILORING		= GetSpellInfo(3908)
local PROF_ALCHEMY			= GetSpellInfo(2259)

local known_professions = {
	[PROF_ALCHEMY]			= false, -- Alchemy
	[PROF_ENCHANTING]		= false, -- Enchanting
	[PROF_JEWELCRAFTING]	= false, -- Jewelcrafting
	[PROF_INSCRIPTION]		= false, -- Inscription
	[PROF_TAILORING]		= false, -- Tailoring
}

local CooldownSpellList = {
-- Initialize my cooldowns list
	-- Alchemy
		-- Northrend Alchemy Research
			[60893] = true,					-- Northrend Alchemy Research
		-- Metals (shared cooldown)
			[11479] = L["Elements"],		-- Transmute: Iron to Gold
			[11480] = L["Elements"],		-- Transmute: Mithril to Truesilver
			-- rm'd in 3.3.0
			-- [60350] = L["Elements"],		-- Transmute: Titanium
		-- Elements (shared cooldown)
			[17559] = L["Elements"],		-- Transmute: Air to Fire
			[17566] = L["Elements"],		-- Transmute: Earth to Life
			[17561] = L["Elements"],		-- Transmute: Earth to Water
			--[[ No cooldown on Elemental Fire
			[25146] = L["Elements"],		
			]]--
			[17560] = L["Elements"],		-- Transmute: Fire to Earth
			[17565] = L["Elements"],		-- Transmute: Life to Earth           
			[17563] = L["Elements"],		-- Transmute: Undeath to Water
			[17562] = L["Elements"],		-- Transmute: Water to Air            
			[17564] = L["Elements"],		-- Transmute: Water to Undeath
			-- Primals
			[28566] = L["Elements"],		-- Transmute: Primal Air to Fire
			[28585] = L["Elements"],		-- Transmute: Primal Earth to Life
			[28567] = L["Elements"],		-- Transmute: Primal Earth to Water
			[28568] = L["Elements"],		-- Transmute: Primal Fire to Earth
			[28583] = L["Elements"],		-- Transmute: Primal Fire to Mana
			[28584] = L["Elements"],		-- Transmute: Primal Life to Earth
			[28582] = L["Elements"],		-- Transmute: Primal Mana to Fire
			[28580] = L["Elements"],		-- Transmute: Primal Shadow to Water
			[28569] = L["Elements"],		-- Transmute: Primal Water to Air
			[28581] = L["Elements"],		-- Transmute: Primal Water to Shadow
			-- Eternals
			[53777] = L["Elements"],		-- Transmute: Eternal Air to Earth
			[52776] = L["Elements"],		-- Transmute: Eternal Air to Water
			[53781] = L["Elements"],		-- Transmute: Eternal Earth to Air
			[53782] = L["Elements"],		-- Transmute: Eternal Earth to Shadow
			[53775] = L["Elements"],		-- Transmute: Eternal Fire to Life
			[53774] = L["Elements"],		-- Transmute: Eternal Fire to Water
			[53773] = L["Elements"],		-- Transmute: Eternal Life to Fire
			[53771] = L["Elements"],		-- Transmute: Eternal Life to Shadow
			[54020] = L["Elements"],		-- Transmute: Eternal Might           
			[53779] = L["Elements"],		-- Transmute: Eternal Shadow to Earth
			[52780] = L["Elements"],		-- Transmute: Eternal Shadow to Life
			[53783] = L["Elements"],		-- Transmute: Eternal Water to Air
			[53784] = L["Elements"],		-- Transmute: Eternal Water to Fire
			-- Epic gems
			[66658] = L["Elements"],		-- Transmute : Ametrine
			[66659] = L["Elements"],		-- Transmute : Cardinal Ruby
			[66660] = L["Elements"],		-- Transmute : King's Amber
			[66662] = L["Elements"],		-- Transmute : Dreadstone
			[66663] = L["Elements"],		-- Transmute : Majestic Zircon
			[66664] = L["Elements"],		-- Transmute : Eye of Zul
		-- Diamonds (shared cooldown)
			--[[ It appears these were removed as well
			[32765] = L["Diamonds"],		-- Transmute: Earthstorm Diamond
			[32766] = L["Diamonds"],		-- Transmute: Skyfire Diamond
			[57425] = L["Diamonds"],		-- Transmute: Skyflare Diamond
			[57427] = L["Diamonds"],		-- Transmute: Earthsiege Diamond
			]]--
	-- Enchanting
		[28027] = true,			-- Prismatic Sphere
		[28028] = true,			-- Void Sphere
    
	-- Jewelcrafting
		[47280] = true,			-- Brilliant Glass    
		[62242] = true,			-- Icy Prism
    
	-- Tailoring
		--[[ These were removed in patch 3.0.8
		[26751] = true,			-- Primal Mooncloth
		[31373] = true,			-- Spellcloth
		[36686] = true,			-- Shadowcloth
		-- and these in 3.3.3
		[56001] = true,			-- Moonshroud
		[56002] = true,			-- Ebonweave
		[56003] = true,			-- Spellweave		]]--
		[56005] = true,			-- Glacial Bag
		-- Cataclysm 
		[75141] = true,			-- Dream of Skywall
		[75142] = true,			-- Dream of Deephome
		[75143] = true,			-- Dreamcloth (Shadow)  
		[75144] = true,			-- Dream of Hyjal
		[75145] = true,			-- Dream of Ragnaros
		[75146] = true,			-- Dream of Azshara
	-- Inscription
		[61288] = true,			-- Minor Inscription Research
		[61177] = true,			-- Northrend Inscription Research

	-- Smelting
	-- killed in 3.3.3
	--	[55208] = true,			-- Smelt Titansteel
}

function BTC_prettyTime( seconds )
	local retString = ""
	local left = seconds
	if ( left > 86400 ) then
		local days = floor( left / 86400 )
		left = left % 86400
		retString = retString .. days .. "d"
	end
	if ( left > 3600 ) then
		local hours = floor( left / 3600 )
		left = left % 3600
		retString = retString .. hours .. "h"
	end
	if ( left > 60 ) then
		local mins = floor( left / 60 )
		retString = retString .. mins .. "m"
	end
	return retString
end

local function SearchForCooldowns()
	addon.Debug( "in SFC" )
	-- This code asswumes a tradeskill window is already open
	-- Thanks to Ackis (ARL) for scanning code.
	for i = GetNumTradeSkills(), 1, -1 do
		local _, tradeType = GetTradeSkillInfo(i)
		if tradeType == "header" then
			ExpandTradeSkillSubClass(i)
		end
	end
	for i = 1, GetNumTradeSkills() do
		local tradeName, tradeType = GetTradeSkillInfo(i)
		-- Ignore all trade skill headers
		if (tradeType ~= "header") then
			-- Get the trade skill link for the specified recipe
			local SpellLink = GetTradeSkillRecipeLink(i)
			-- Get the SpellID from the spell link
			if ( SpellLink ) then 
				local SpellID = tonumber( string.match( SpellLink, "enchant:(%d+)" ) )
				-- Is this a spell I track the cooldown of?
				if ( CooldownSpellList[SpellID] ~= nil ) then
					-- Check if there is a current cooldown
					local cooldown = GetTradeSkillCooldown(i)
					if ( cooldown == nil ) then
						addon.Debug( tradeName .. ", cd is nil" )
						db.cooldowns[SpellID] = {}
						db.cooldowns[SpellID].endtime = 0
						if ( CooldownSpellList[SpellID] ~= true ) then
							db.cooldowns[SpellID].displayName = CooldownSpellList[SpellID]
						else
							db.cooldowns[SpellID].displayName = tradeName
						end
					else
						addon.Debug( tradeName .. ", cd is " .. cooldown )
						local cd = floor( cooldown )
						currtime = time()
						db.cooldowns[SpellID] = {}
						db.cooldowns[SpellID].endtime = currtime + cd
						if ( CooldownSpellList[SpellID] ~= true ) then
							db.cooldowns[SpellID].displayName = CooldownSpellList[SpellID]
						else
							db.cooldowns[SpellID].displayName = tradeName
						end
					end
				else
					-- Its possible we have an entry for a cooldown that was
					-- removed from the game. Lets check for it. If its there,
					-- remove it
					if ( db.cooldowns[SpellID] ~= nil ) then
						db.cooldowns[SpellID] = nil
					end
				end
			end
		end
	end
end

local function InitiateTradeScan()
	addon.Debug( "In ITS" )
	-- check to see which tradeskills we know, and only professions that have cooldown...
	-- Thanks to Torhal (Revelation) for this code. Used with permission. And especially since
	-- he robbed it from AckisRecipeList to begin with...
	-- Reset the table, they may have unlearnt a profession - I robbed Ackis!
	for i in pairs(known_professions) do
		known_professions[i] = false
	end
	-- Grab names from the spell book
	for index = 1, 25, 1 do
		local spell_name = GetSpellName(index, BOOKTYPE_SPELL)

		if not spell_name or (index == 25) then
			break
		end

		if known_professions[spell_name] == false then
			addon.Debug( "I know " .. spell_name )
			known_professions[spell_name] = true
		end
	end
	-- disable the sound of opening the tradeskill window...
	local sfx = tonumber(GetCVar("Sound_EnableSFX"))
	SetCVar("Sound_EnableSFX", 0)
	-- scan known
	ForcedScan = true
	for prof, known in pairs(known_professions) do
		if known then
			addon.Debug( "Scanning " .. prof )
			CastSpellByName( prof )
			SearchForCooldowns()
			CloseTradeSkill()
		end
	end
	ForcedScan = false
	SetCVar("Sound_EnableSFX", sfx)
end

function BTC:BAG_UPDATE( ... )
	-- I don't actually care about what updated. Just that it did. I only turn on monitoring
	-- for this event when a SPELL_CAST_START has happened for a spell (tradeskill) that has a
	-- cooldown has occurred. And only when it isn't an enchant.
	addon.Debug( "BAG_UPDATE" )
	BTC:UnregisterEvent( "BAG_UPDATE" )
	-- scan skills again to detect a new cooldown
	if ( TradeSkillOpen == false ) then
		addon.Debug( "ITS" )
		InitiateTradeScan()
	else
		addon.Debug( "SFC" )
		SearchForCooldowns()
	end
	addon.updateText()
end

function BTC:ENCHANT_APPLIED( ... )
	-- Same thing here. I only turn on monitoring for this when I see a SPELL_CAST_START for
	-- and enchant that I'm casting.
end

function BTC:TRADE_SKILL_SHOW( eventName, ... )
	TradeSkillOpen = true
	addon.Debug( "In " .. eventName )
	-- if I forced this scan, don't do anything
	if ( ForcedScan ~= true ) then
		-- check to see if this is ours or someone else's
		isLink = IsTradeSkillLinked()
		if ( isLink == nil ) then
			-- Thanks to Ackis (ARL) for scanning code.
			for i = GetNumTradeSkills(), 1, -1 do
				local _, tradeType = GetTradeSkillInfo(i)
				if tradeType == "header" then
					ExpandTradeSkillSubClass(i)
				end
			end
			for i = 1, GetNumTradeSkills() do
				local tradeName, tradeType = GetTradeSkillInfo(i)
				-- Ignore all trade skill headers
				if (tradeType ~= "header") then
					-- Get the trade skill link for the specified recipe
					local SpellLink = GetTradeSkillRecipeLink(i)
					-- Get the SpellID from the spell link
					if ( SpellLink ) then 
						local SpellID = tonumber( string.match( SpellLink, "enchant:(%d+)" ) )
						-- Is this a spell I track the cooldown of?
						if ( CooldownSpellList[SpellID] ~= nil ) then
							-- Check if there is a current cooldown
							local cooldown = GetTradeSkillCooldown(i)
							if ( cooldown == nil ) then
								addon.Debug( tradeName .. ", cd is nil" )
								db.cooldowns[SpellID] = {}
								db.cooldowns[SpellID].endtime = 0
								if ( CooldownSpellList[SpellID] ~= true ) then
									db.cooldowns[SpellID].displayName = CooldownSpellList[SpellID]
								else
									db.cooldowns[SpellID].displayName = tradeName
								end
							else
								addon.Debug( tradeName .. ", cd is " .. cooldown )
								local cd = floor( cooldown )
								currtime = time()
								db.cooldowns[SpellID] = {}
								db.cooldowns[SpellID].endtime = currtime + cd
								if ( CooldownSpellList[SpellID] ~= true ) then
									db.cooldowns[SpellID].displayName = CooldownSpellList[SpellID]
								else
									db.cooldowns[SpellID].displayName = tradeName
								end
							end
						else
							-- Its possible we have an entry for a cooldown that was
							-- removed from the game. Lets check for it. If its there,
							-- remove it
							if ( db.cooldowns[SpellID] ~= nil ) then
								db.cooldowns[SpellID] = nil
							end
						end
					end
				end
			end
		else
			addon.Debug( "Someone else's tradeskill" )
		end
		addon.updateText()
	end
end

function BTC:TRADE_SKILL_CLOSE( ... )
	TradeSkillOpen = false
end

function BTC:COMBAT_LOG_EVENT_UNFILTERED( _, _, eventType, _, srcName, _, _, _, _, spellId, ...)
	local now = time()
    if ( ( srcName ~= nil ) and
		 ( srcName == BTC.me ) and 
		 ( eventType == "SPELL_CAST_START" ) ) then

		local dbc = db.cooldowns
		-- First, handle the special case items with cooldowns
		--[[ Removed at some point...
		if ( spellId == 19566 ) then
			dbc[spellId].displayName = L["Salt Shaker"]
			dbc[spellId].endtime = now + 252000 ]]--
		if ( spellId == 21935 ) then					-- Snowmaster 9000
			dbc[spellId].displayName = L["Snowmaster 9000"]
			dbc[spellId].endtime = now + 72000
		elseif ( spellId == 26265 ) then					-- Create Elune Stone
			dbc[spellId].displayName = L["Elune's Lantern"]
			dbc[spellId].endtime = now + 72000
		elseif ( CooldownSpellList[spellId] ~= nil ) then
			addon.Debug( L["DETECTED_SPELLCAST"] )
			-- Right now, all spells with cooldowns place a new item into your bags. Right at the second this
			-- code executes, the player will still be in mid-spell-cast of the spell in question. So in order
			-- to determine when to check for a new cooldown, I'm now going to register for a BAG_UPDATE (the
			-- item hitting my bags), and then scan for new skills.
			BTC:RegisterEvent( "BAG_UPDATE" )
		else
			addon.Debug( "Detected spellcast that wasn't in list. spellid = " .. spellId )
		end
	end
end

function BTC_checkSkills()
	local numSkills = GetNumSkillLines()
	for i = numSkills, 1, -1 do
		local _, isHeader = GetSkillLineInfo(i)
		if ( isHeader ~= nil ) then
			ExpandSkillHeader(i)
		end
	end
	numSkills = GetNumSkillLines()
	addon.Debug( "numskills = " .. numSkills )
	for i = 1, numSkills do
		local skillName, isHeader, _, skillRank, _, _, skillMaxRank = GetSkillLineInfo(i)
		-- Ignore all trade skill headers
		if ( isHeader == nil ) then
			if ( primarySkills[ skillName ] == true ) then
				db.primary[ skillName ] = {}
				db.primary[ skillName ].currlevel = skillRank
				db.primary[ skillName ].maxlevel = skillMaxRank
			elseif ( secondarySkills[ skillName] == true ) then
				db.secondary[ skillName ] = {}
				db.secondary[ skillName ].currlevel = skillRank
				db.secondary[ skillName ].maxlevel = skillMaxRank
			elseif ( weaponSkills[ skillName ] == true ) then
				db.weapons[ skillName ] = {}
				db.weapons[ skillName ].currlevel = skillRank
				db.weapons[ skillName ].maxlevel = skillMaxRank
			else
				addon.Debug( string.format( L["Unknown skill : %s"], skillName ) )
			end
		end
	end
end

local function BTC_MouseHandler( cell, arg, button )
	addon.Debug( "Received .. " .. arg ) 
	-- only react to Alt-Leftclick
	if ( button == "LeftButton" ) then
		if ( IsAltKeyDown() ) then
		-- do resetting stuff
		addon.Debug( "alt-left-click received" )
		end
	end
end

local function DrawTooltip()
	tooltip:Hide()
	tooltip:Clear()

	local dbf = BTC.db.factionrealm
	local lineNum = 0

	-- Show my primary skills
	tooltip:AddHeader( White( L["Professions :"] ) )
	for k, v in pairs( db.primary ) do
		if ( v.currlevel == v.maxlevel ) then
			tooltip:AddLine( Yellow( "   " .. k ), " ", Yellow( "(" .. v.currlevel .. "/" .. v.maxlevel .. ")" ) )
		else
			tooltip:AddLine( Yellow( "   " .. k ), " ",
				Yellow("(")..Red(v.currlevel)..Yellow("/")..Green(v.maxlevel)..Yellow(")") )
		end
	end
	-- Show Secondary
	if ( db.options.showSecondary == true ) then
		tooltip:AddLine( " " )
		tooltip:AddHeader( White( L["Secondary Skills :"] ) )
		for k, v in pairs( db.secondary ) do
			if ( v.currlevel == v.maxlevel ) then
				tooltip:AddLine( Yellow( "   " .. k ), " ", Yellow( "(" .. v.currlevel .. "/" .. v.maxlevel .. ")" ) )
			else
				tooltip:AddLine( Yellow( "   " .. k ), " ",
					Yellow("(")..Red(v.currlevel)..Yellow("/")..Green(v.maxlevel)..Yellow(")") )
			end
		end
	end
	-- Show Weapon
	if ( db.options.showWeapons == true ) then
		tooltip:AddLine( " " )
		tooltip:AddHeader( White( L["Weapon Skills :"] ) )
		for k, v in pairs( db.weapons ) do
			if ( v.currlevel == v.maxlevel ) then
				tooltip:AddLine( Yellow( "   " .. k ), " ", Yellow( "(" .. v.currlevel .. "/" .. v.maxlevel .. ")" ) )
			else
				tooltip:AddLine( Yellow( "   " .. k ), " ",
					Yellow("(")..Red(v.currlevel)..Yellow("/")..Green(v.maxlevel)..Yellow(")") )
			end
		end
	end
	-- cooldowns
	tooltip:AddLine( " " )
	local Displayed = {}
	local mouseString = ""
	tooltip:AddHeader( White( L["Cooldowns :"] ) )
	for k, v in pairs( dbf ) do
		if ( next( v.cooldowns ) ~= nil ) then
			lineNum = tooltip:AddLine( Yellow( "   " .. k ) )
			mouseString = "n~" .. k
			tooltip:SetLineScript( lineNum, "OnMouseDown", BTC_MouseHandler, mouseString )
		end
		for i, a in pairs( v.cooldowns ) do
			local blort = k .. "-" .. a.displayName
			if not ( Displayed[blort] ) then
				Displayed[blort] = true
				local now = time()
				local timeleft = a.starttime
				if ( ( a.endtime == 0 ) or ( now > a.endtime ) ) then
					lineNum = tooltip:AddLine( Yellow( "      " .. a.displayName ), " ", Green( L["Ready"] ) )
				else
					local pretty = BTC_prettyTime( a.endtime - now )
					lineNum = tooltip:AddLine( Yellow( "      " .. a.displayName ), " ", Red( pretty ) )
				end
				mouseString = "s~" .. i
				tooltip:SetLineScript( lineNum, "OnMouseDown", BTC_MouseHandler, mouseString )
			end
		end
	end
	-- Footer
	tooltip:AddLine(" ")
	lineNum = tooltip:AddLine( "b" )
	tooltip:SetCell( lineNum, 1, L["Click for Configuration and Help"], "LEFT", 3 )
end

local LDB = LibStub( "LibDataBroker-1.1", true )
local launcher = LDB:NewDataObject( "BTC", {
	type = "data source",
	text = "",
	label = "Broker_TradeCooldowns",
	icon = "Interface\\Addons\\Broker_TradeCooldowns\\icon",
	OnClick = function(clickedframe, button)
				BTC:ShowConfig()
				addon.updateText()
	end,
	OnEnter = function ( self )
		tooltip = LTT:Acquire( MODNAME.."TT", 3, "LEFT", "CENTER", "RIGHT" ) -- or GameTooltip
		tooltip:SetScale( db.options.tipscale )
		DrawTooltip()
		tooltip:SetAutoHideDelay( .1, self )
		tooltip:EnableMouse()
		tooltip:SmartAnchorTo( self )
		tooltip:Show()
	end,
	OnLeave = function ( )
		-- this should get handled by the autohide delay stuff now
	end,
} )

-- options stuff
local displayOptions = nil
local function giveDisplay()
	if (not displayOptions) then
		displayOptions = {
			order = 4,
			type = "group",
			name = L["Display Options"],
			desc = L["DISPLAY_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 41,
					type	= "header",
					name	= L["Display Options"],
				},
				desc = {
					order	= 42,
					type	= "description",
					name	= L["DISPLAY_OPTIONS_DESC"],
				},
				tooltipscale = {
					order	= 43,
					type	= "range",
					name	= L["Tooltip Scale"],
					desc	= L["TOOLTIP_SCALE_DESC"],
					min		= .7,
					max		= 1.3,
					step	= .05,
					bigStep = .05,
					get		= function() return db.options.tipscale end,
					set		= function( info, v ) 
								db.options.tipscale = v
								tooltip:SetScale( v )
							  end,
				},
				debug = {
					order	= 44,
					type	= "toggle",
					name	= L["Debug"],
					desc	= L["DEBUG_DESC"],
					get		= function() return db.options.debug end,
					set		= function() db.options.debug = not db.options.debug end,
				},
				showweapons = {
					order	= 45,
					type	= "toggle",
					name	= L["Show Weapons"],
					desc	= L["SHOW_WEAPONS_DESC"],
					get		= function() return db.options.showWeapons end,
					set		= function() db.options.showWeapons = not db.options.showWeapons end,
				},
				showsecondary = {
					order	= 46,
					type	= "toggle",
					name	= L["Show Secondary"],
					desc	= L["SHOW_SECONDARY_DESC"],
					get		= function() return db.options.showSecondary end,
					set		= function() db.options.showSecondary = not db.options.showSecondary end,
				},
				showtext = {
					order	= 50,
					type	= "toggle",
					name	= L["Show Text"],
					desc	= L["SHOW_TEXT_DESC"],
					get		= function() return db.options.showText end,
					set		= function()
						db.options.showText = not db.options.showText
						addon.updateText()
					end,
				},
				textcooldown = {
					order	= 51,
					type	= "select",
					name	= L["Select Cooldown to Display"],
					desc	= L["SHOW_TEXTCD_DESC"],
					get		= function() return db.options.textCooldown end,
					set		= function( info, value )
						db.options.textCooldown = value
						addon.updateText()
					end,
					values	= function()
						local t = {}
						if ( next( db.cooldowns ) ~= nil ) then
							for k, v in pairs( db.cooldowns ) do
								t[k] = v.displayName
							end
						end
						return t
					end,
					disabled	= function()
						if ( db.options.showText ) then
							if ( next( db.cooldowns ) ~= nil ) then
								return false
							else
								return true
							end
						else
							return true
						end
					end,
				},
				header2 = {
					order	= 91,
					type	= "header",
					name	= L["Help and Usage Notes"],
				},
				help1 = {
					order	= 92,
					type	= "description",
					name	= "- " .. L["LDB text will update approximately once a minute"],
				},
				help2 = {
					order	= 93,
					type	= "description",
					name	= "- " .. L["Alt-Left-Clicking a cooldown or player will remove it from the saved data"],
				},
				help3 = {
					order	= 94,
					type	= "description",
					name	= "- " .. L["Alt-Right-Clicking a cooldown will 'hide' it from view (it will still be in the saved data, however)"],
				},
				help4 = {
					order	= 95,
					type	= "description",
					name	= "- " .. L["Alt-Right-Clicking a player will 'unhide' any hidden cooldowns for that player"],
				},
			},
		}
	end
	return displayOptions
end

local modularOptions = {}
local options = nil
local function fullOptions()
	if (not options) then
		options = {
			type = "group",
			name = MODNAME,
			args = {
				general = {
					order	= 1,
					type	= "group",
					name	= L["General Information"],
					desc	= L["GENERAL_INFO_DESC"],
					args	= {
						header1 = {
							order	= 11,
							type	= "header",
							name	= L["General Information"],
						},
						desc = {
							order	= 12,
							type	= "description",
							name	= L["GENERAL_INFO_DESC"],
						},
						author = {
							order	= 13,
							type	= "description",
							name	= L["Author : "] .. addonauthor .. "\n",
						},
						version = {
							order	= 14,
							type	= "description",
							name	= L["Version : "] .. addonversion .. "\n",
						},
						builddate = {
							order	= 15,
							type	= "description",
							name	= L["Build Date : "] .. builddate .. "\n",
						},
						dbversion = {
							order	= 15,
							type	= "description",
							name	= L["Database Version : "] .. db.options.dbVersion .. "\n",
						},
						translators = {
							order	= 16,
							type	= "description",
							name	= string.format( L["Helpful Translators (thank you) : %s"],
										"whynona2(frFR), SunTsu(deDE), Dessa(deDE), " ..
										"amunra(koKR), xska(ruRU), <CWDG>tnt2ray(zhCN), " ..
										"NightOw1(zhTW), andy52005(zhTW)\n" )
						},
					},
				},
			},
		}
	end
	
	for k,v in pairs(modularOptions) do
		options.args[k] = (type(v) == "function") and v() or v
	end

	return options
end

function addon:SetupOptions()
	AceConfigReg:RegisterOptionsTable(MODNAME, fullOptions)
	self.optionsFrame = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")

	-- Fill up our modular options...
	self:RegisterModuleOptions("Display", giveDisplay(), L["Display Options"])
end

function addon:RegisterModuleOptions(name, optionsTable, displayName)
	modularOptions[name] = optionsTable
	self.optionsFrame[name] = AceConfigDialog:AddToBlizOptions(MODNAME, displayName, MODNAME, name)
end

function BTC:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory( self.optionsFrame["Display"] )
end

function BTC.updateText()
	addon.Debug( "In updateText" )
	local newStr = " "
	if ( db.options.showText ) then
		if ( db.options.textCooldown ~= 0 ) then
			local entry = db.cooldowns[ db.options.textCooldown]
			if ( entry ) then
				newStr = Yellow( entry.displayName ) .. ":"
				local now = time()
				if ( ( entry.endtime == 0 ) or ( now > entry.endtime ) ) then
					newStr = newStr .. Green( L["Ready"] )
				else
					local pretty = BTC_prettyTime( entry.endtime - now )
					newStr = newStr .. Red( pretty )
				end
			end
		end
	end
	launcher.text = newStr
end

function BTC_upgradeDB()
	local olddb = BTC.db.factionrealm[BTC.me]
	-- This checks the current DB version the person has, and upgrades as necessary
	if not ( olddb.options ) then
		-- They have the original version
		olddb.options = {}
		olddb.options = {
			dbVersion = "1.0",
			debug = false,
			showWeapons = false,
			showSecondary = false,
			showClass = false,
			showArmor = false,
			showLanguages = false,
		}
		olddb.weapons = {}
		olddb.primary = {}
		olddb.secondary = {}
		olddb.class = {}
		olddb.languages = {}
		olddb.armor = {}
		olddb.skills = nil
		addon.Print( string.format( L["Database upgraded to %s"], "1.0" ) )
	end
	if ( olddb.options.dbVersion == "1.0" ) then
		olddb.options.tipscale = 1
		olddb.options.dbVersion = "1.1"
		addon.Print( string.format( L["Database upgraded to %s"], "1.1" ) )
	end
	if ( olddb.options.dbVersion == "1.1" ) then
		olddb.options.showText = false
		olddb.options.textCooldown = 0
		olddb.options.dbVersion = "1.2"
		addon.Print( string.format( L["Database upgraded to %s"], "1.2" ) )
	end
	if ( olddb.options.dbVersion == "1.2" ) then
		olddb.options.showClass = nil
		olddb.options.showArmor = nil
		olddb.options.showLanguages = nil
		olddb.languages = nil
		olddb.class = nil
		olddb.armor = nil
		olddb.options.dbVersion = "1.3"
		addon.Print( string.format( L["Database upgraded to %s"], "1.2" ) )
	end
	if ( olddb.options.dbVersion == "1.3" ) then
		-- clear out old cooldowns from the cooldown list
		for k, v in pairs( olddb.cooldowns ) do
			if ( CooldownSpellList[k] == nil ) then
				olddb.cooldowns[k] = nil
			end
		end
		olddb.options.dbVersion = "1.4"
		addon.Print( string.format( L["Database upgraded to %s"], "1.4" ) )
	end
end

local function stopScanning()
	if (BTC_ScanFrame) then
		BTC_ScanFrame:SetScript("OnUpdate", nil);
	end
end

function addon.scanSkills()
	-- on first load, there's no real good way to check if the player's skills
	-- are available for scanning yet. So just scan every 5 seconds until they are
	if (not BTC_ScanFrame) then
		BTC_ScanFrame = CreateFrame("Frame")
	end

	local totalElapsed = 0
	local checkInterval = 5
	local gotFirst = false

	BTC_ScanFrame:SetScript("OnUpdate", function(self, elapsed)
		totalElapsed = totalElapsed + elapsed
		if ( totalElapsed < checkInterval ) then
			return;
		end
		totalElapsed = 0;
		
		if not ( gotFirst ) then
			local numSkills = GetNumSkillLines()
			if ( numSkills ~= 0 ) then
				BTC_checkSkills()
				addon.updateText()
				checkInterval = 60
				gotFirst = true
			end
		else
			BTC_checkSkills()
			addon.updateText()
		end
		return
	end )
end

-- on init
function BTC:OnInitialize()
end

function BTC:OnEnable()
    -- Called when the addon is loaded
    self.db = LibStub( "AceDB-3.0" ):New( "BTC_DB", nil, "Default" )
	if ( not self.db.factionrealm) then
		self.db.factionrealm = {}
	end

	BTC.me = UnitName( "player" )

	-- do we have any data yet for this toon? If not, init its table
	if not ( self.db.factionrealm[BTC.me] ) then
		self.db.factionrealm[BTC.me] = {
			primary = {},
			secondary = {},
			weapons = {},
			class = {},
			languages = {},
			armor = {},
			cooldowns = {},
			options = {
				dbVersion = "1.3",
				debug = false,
				showSecondary = false,
				showWeapons = false,
				showText = false,
				textCooldown = 0,
				tipscale = 1,
			}
		}
	else
		-- check if we need to upgrade the db
		BTC_upgradeDB()
	end

	db = self.db.factionrealm[BTC.me]

	addon.Debug( "In OnEnable" )

	BTC:SetupOptions()

	-- Check current weapons/languages
	addon.scanSkills()
	addon.updateText()

	-- Event listeners
	BTC:RegisterEvent( "TRADE_SKILL_SHOW" )
	BTC:RegisterEvent( "TRADE_SKILL_CLOSE" )
	BTC:RegisterEvent( "COMBAT_LOG_EVENT_UNFILTERED" )
	BTC:RegisterEvent( "CHAT_MSG_SKILL", BTC_checkSkills )
end

--[[
************************************************************************
CHANGELOG:
Date : 9/12/10
	Bumped TOC for 30300 and added in X-Compatible-With 40000
	Removed SpellWeave/Moonshroud/etc
	Added Glacial Bag and several Cata cloth CDs
	Removing stuff noone cares about (armor,language,class)
	lots of code cleanup
	Added Epic Gem Transmutes
	Properly detect new cooldown
	Properly update LDB text
Date : 3/28/10
	Removed cooldowns that were removed from game up to 3.3.3
Date : 3/16/09
	Removed cooldowns that were removed from game
	added check for invalid spelllinks
	added credits for zhTW, deDE
	added removal from the list of no longer available cooldowns
Date : 11/14/08
	IsTradeSkillLinked code (check to see if the tradeskill open is our
	own or someone elses)
	Allow the skills listed to update
	added languages and weapons
	added db_upgrade
	added event for skillups
	added libtooltip
	added tipscale option
Date : 10/29/08
	Initial version
************************************************************************
]]--